--
-- BaleWrapperExtension
--
-- @author:    	Xentro (Marcus@Xentro.se)
-- @website:	www.Xentro.se
-- @history:	v0.62 - 2015-08-08 - 
-- 
--[[
<BaleWrapperExtension delayTimer="7000" >
	<balerDelaySound file="" pitchOffset="1" volume="1" />
</BaleWrapperExtension>
]]--

BaleWrapperExtension = {};

function BaleWrapperExtension.prerequisitesPresent(specializations)
    return SpecializationUtil.hasSpecialization(Baler, specializations) and SpecializationUtil.hasSpecialization(BaleWrapper, specializations);
end;

function BaleWrapperExtension:load(xmlFile)
	self.setWrappingActive = SpecializationUtil.callSpecializationsFunction("setWrappingActive");
	self.isUnloadingAllowed = Utils.overwrittenFunction(self.isUnloadingAllowed, BaleWrapperExtension.isUnloadingAllowed);
	self.doStateChange = Utils.overwrittenFunction(self.doStateChange, BaleWrapperExtension.doStateChange);
	
	if self.isClient then
		self.balerDelaySound = Utils.loadSample(xmlFile, {}, "vehicle.BaleWrapperExtension.balerDelaySound", nil, self.baseDirectory);
	end;
	
	self.emptyBalerDelay = {false, 0, Utils.getNoNil(getXMLFloat(xmlFile, "vehicle.BaleWrapperExtension#delayTimer"), 7000)};
	self.wrappingActive = false;
	self.forceWrappingStop = false;
	
	self.hasBaleWrapperExtension = true;
	self.allowBalesFromOutside = false;
	
	local fillType = Fillable.fillTypeNameToInt["silage"];
	
	if self.allowedBaleTypes[fillType] == nil then
		self.allowedBaleTypes[fillType] = {}; -- fast workaround to allow silage for stop wrapping feature
	end;
end;

function BaleWrapperExtension:delete()
	if self.isClient then
		Utils.deleteSample(self.balerDelaySound);
	end;
end;

function BaleWrapperExtension:readStream(streamId, connection)
	self:setWrappingActive(streamReadBool(streamId), true);
end;

function BaleWrapperExtension:writeStream(streamId, connection)
	streamWriteBool(streamId, self.wrappingActive);
end;

function BaleWrapperExtension:loadFromAttributesAndNodes(xmlFile, key, resetVehicles)
	if not resetVehicles then
		local wrappingState = getXMLBool(xmlFile, key .. "#wrappingActive");
		if wrappingState ~= nil then
			self:setWrappingActive(wrappingState, true);
		end;
	end;
	
	return BaseMission.VEHICLE_LOAD_OK;
end;

function BaleWrapperExtension:getSaveAttributesAndNodes(nodeIdent)
	local attributes = 'wrappingActive="' .. tostring(self.wrappingActive) .. '"';
	
	return attributes;
end;

function BaleWrapperExtension:mouseEvent(posX, posY, isDown, isUp, button)
end;

function BaleWrapperExtension:keyEvent(unicode, sym, modifier, isDown)
end;

function BaleWrapperExtension:update(dt)
	if self:getIsActive() then
		if self:getIsActiveForInput() then
			if InputBinding.hasEvent(InputBinding.IMPLEMENT_EXTRA4) then
				if not self.forceWrappingStop then
					if self.balerUnloadingState == Baler.UNLOADING_CLOSED then
						if self.baleWrapperState ~= BaleWrapper.STATE_WRAPPER_WRAPPING_BALE then
							self:setWrappingActive(not self.wrappingActive);
						else
							g_currentMission:showBlinkingWarning(g_i18n:getText("BWE_ERROR_WRAPPING"), 1000);
						end;
					else
						g_currentMission:showBlinkingWarning(g_i18n:getText("BWE_ERROR_UNLOADING"), 1000);
					end;
				else
					if self.forcedWrappingStoppedText ~= nil then
						g_currentMission:showBlinkingWarning(self.forcedWrappingStoppedText);
					end;
				end;
			end;
		end;
	end;
end;

function BaleWrapperExtension:updateTick(dt)
	if self:getIsActive() then
		if self.balerUnloadingState == Baler.UNLOADING_CLOSED then
			if table.getn(self.bales) > 0 then
				self.emptyBalerDelay[1] = true;
			end;
		elseif self.balerUnloadingState == Baler.UNLOADING_OPEN  then
			self:setIsUnloadingBale(false, true);
		end;
		
		if self.isTurnedOn then 
			if self.emptyBalerDelay[1] then
				if self.emptyBalerDelay[2] >= self.emptyBalerDelay[3] then
					if self.isClient then
						if self.balerDelaySound.isPlaying then
							Utils.stopSample(self.balerDelaySound, true);
						end;
					end;
					
					if self:isUnloadingAllowed() then
						self.emptyBalerDelay[1] = false;
						self.emptyBalerDelay[2] = 0;
						self:setIsUnloadingBale(true, true);
					end;
				else
					if self.isClient then
						if not self.balerDelaySound.isPlaying and self:getIsActiveForSound() then
							Utils.playSample(self.balerDelaySound, 0, 0, nil);
						end;
					end;
					
					self.emptyBalerDelay[2] = self.emptyBalerDelay[2] + dt;
				end;
			end;
		else
			if self.isClient then
				if self.balerDelaySound.isPlaying then
					Utils.stopSample(self.balerDelaySound, true);
				end;
			end;
		end;
	end;
end;

function BaleWrapperExtension:draw()
	if self.balerDelaySound.isPlaying then
		g_currentMission:addExtraPrintText(g_i18n:getText("BWE_HOLD"));
	end;
	
	if self.balerUnloadingState == Baler.UNLOADING_CLOSED and not self.forceWrappingStop then
		if self.baleWrapperState ~= BaleWrapper.STATE_WRAPPER_WRAPPING_BALE then
			local txt = g_i18n:getText("BWE_DEACTIVE");
			if self.wrappingActive then
				txt = g_i18n:getText("BWE_ACTIVE");
			end;
			
			g_currentMission:addHelpButtonText(string.format(g_i18n:getText("BWE_CURRENT_STATE"), txt), InputBinding.IMPLEMENT_EXTRA4);
		end;
	end;
end;

function BaleWrapperExtension:onLeave()
	if self.deactivateOnLeave then
		BaleWrapperExtension.onDeactivate(self);
	else
		BaleWrapperExtension.onDeactivateSounds(self);
	end;
end;

function BaleWrapperExtension:onDetach()
	if self.deactivateOnDetach then
		BaleWrapperExtension.onDeactivate(self);
	else
		BaleWrapperExtension.onDeactivateSounds(self);
	end;
end;

function BaleWrapperExtension:onDeactivate()
	BaleWrapperExtension.onDeactivateSounds(self);
end;

function BaleWrapperExtension:onDeactivateSounds()
	if self.isClient then
		Utils.stopSample(self.balerDelaySound, true);
	end;
end;

function BaleWrapperExtension:setWrappingActive(state, noEventSend)
	self.wrappingActive = state;
	
	if state then
		local bale = networkGetObject(self.wrapper.currentBale);
		
		if bale ~= nil then
			if bale.supportsWrapping then
				local baleTypes = self.allowedBaleTypes[bale:getFillType()];
				
				if baleTypes ~= nil then
					if bale.wrappingState == 0 then
						self:moveBaleToWrapper(bale);
						-- self:doStateChange(BaleWrapper.CHANGE_WRAPPING_START);
					end;
				end;
			end;
		end;
	end;
	
	setWrappingActiveEvent.sendEvent(self, state, noEventSend);
end;

function BaleWrapperExtension:isUnloadingAllowed(func)
	local allowed = false;
	
	if func ~= nil then
		allowed = func(self);
	end;
	
	if self.balerDelaySound.isPlaying and allowed then
		allowed = false;
	end;
	
	return allowed;
end;

function BaleWrapperExtension:doStateChange(func, id, nearestBaleServerId)
	local currentBale, baleAllowed, keepNoneSupportedBale, isWrapped = networkGetObject(self.wrapper.currentBale), true, false, false;
	
	if currentBale ~= nil then
		-- we need to get fillType before it move bale to wrapper..
		local baleType = self.allowedBaleTypes[currentBale:getFillType()];
		isWrapped = currentBale.wrappingState == 1;
		
		if baleType == nil then
			baleAllowed = false;
		end;
	end;
	
	if (id == BaleWrapper.CHANGE_WRAPPER_START_DROP_BALE and self.baleWrapperState == BaleWrapper.STATE_MOVING_GRABBER_TO_WORK and not baleAllowed) -- none supported bales
	or (id == BaleWrapper.CHANGE_WRAPPING_START and not self.wrappingActive) -- wrapper deactivated
	or (id == BaleWrapper.CHANGE_WRAPPING_START and self.wrappingActive and not baleAllowed) then -- wrapper active but bale not allowed
		keepNoneSupportedBale = true;
		id = BaleWrapper.CHANGE_WRAPPING_BALE_FINSIHED;
	end;
		
	if id == BaleWrapper.CHANGE_DROP_BALE_AT_GRABBER then
		local attachNode = self.wrapper.baleNode;
		local bale = currentBale;
		if bale ~= nil then
			bale:mount(self, attachNode, 0, 0, 0, 0, 0, 0);
			self.baleToMount = nil;
			
			if self.wrappingActive then
				if self.allowedBaleTypes[bale:getFillType()] ~= nil then
					self:updateWrapNodes(true, false, 0);
				end;
			end;
		else
			self.baleToMount = {serverId = self.baleGrabber.currentBale, linkNode = attachNode, trans = {0, 0, 0}, rot = {0, 0, 0}};
			bale = networkGetObject(self.baleGrabber.currentBale);
		end;
		
		
		self.wrapper.currentBale = self.baleGrabber.currentBale;
		self.baleGrabber.currentBale = nil;

		self:playAnimation("moveBaleToWrapper", -1, nil, true);
		self.baleWrapperState = BaleWrapper.STATE_MOVING_GRABBER_TO_WORK;
		
	elseif id == BaleWrapper.CHANGE_WRAPPING_BALE_FINSIHED then
		if not keepNoneSupportedBale then
			if self.isClient then
				Utils.stopSample(self.wrapperSound);
				
				if self.isClient and self:getIsActiveForSound() then
					Utils.playSample(self.wrapperStopSound, 1, 0);
				end;
			end;
			
			self:updateWrappingState(1, true);
		end;
		
		self.baleWrapperState = BaleWrapper.STATE_WRAPPER_FINSIHED;
	
	elseif id == BaleWrapper.CHANGE_WRAPPER_START_DROP_BALE then
		if isWrapped then
			self:updateWrapNodes(false, false, 0);
		end;
		
		self:playAnimation("dropBaleFromWrapper", 1, nil, true);
		self.baleWrapperState = BaleWrapper.STATE_WRAPPER_DROPPING_BALE;
	else
		if func ~= nil then
			func(self, id, nearestBaleServerId);
		end;
	end;
end;


-- Fix
if not Utils.BaleWrapperExtensionFix20150317 then
	Utils.BaleWrapperExtensionFix20150317 = true;
	
	local oldGetBaleInRange = BaleWrapper.getBaleInRange;
	BaleWrapper.getBaleInRange = function(self, grabNode)
		if self.hasBaleWrapperExtension ~= nil and self.hasBaleWrapperExtension and not self.allowBalesFromOutside then
			return nil; -- don't grab bales from outside!
		else
			return oldGetBaleInRange(self, grabNode);
		end;
	end;
	
	local oldGetSaveAttributesAndNodes = BaleWrapper.getSaveAttributesAndNodes;
	BaleWrapper.getSaveAttributesAndNodes = function(self, nodeIdent)
		local attributes, nodes = oldGetSaveAttributesAndNodes(self, nodeIdent);
		
		if attributes ~= nil then
			attributes = attributes:gsub("fillLevel", "wrapperFillLevel");
		end;
		
		return attributes, nodes;
	end;
	
	local oldLoadFromAttributesAndNodes = BaleWrapper.loadFromAttributesAndNodes;
	BaleWrapper.loadFromAttributesAndNodes = function(self, xmlFile, key, resetVehicles)
		local state = oldLoadFromAttributesAndNodes(self, xmlFile, key, resetVehicles);
		
		if not resetVehicles then
			if self.baleToLoad ~= nil then
				self.baleToLoad.fillLevel = getXMLFloat(xmlFile, key .. "#wrapperFillLevel"); -- get correct fillLevel
			end;
		end;
		
		return state;
	end;
	
	local oldPickupWrapperBale = BaleWrapper.pickupWrapperBale;
	BaleWrapper.pickupWrapperBale = function(self, bale, baleType)
		
		if self.hasBaleWrapperExtension ~= nil and self.hasBaleWrapperExtension then
			if self.wrappingActive then
				oldPickupWrapperBale(self, bale, baleType);
			else
				-- just grab current bale
				g_server:broadcastEvent(BaleWrapperStateEvent:new(self, BaleWrapper.CHANGE_GRAB_BALE, networkGetObjectId(bale)), true, nil, self);
			end;
		else
			oldPickupWrapperBale(self, bale, baleType);
		end;
	end;
end;


-- Event
setWrappingActiveEvent = {};
setWrappingActiveEvent_mt = Class(setWrappingActiveEvent, Event);

InitEventClass(setWrappingActiveEvent, "setWrappingActiveEvent");

function setWrappingActiveEvent:emptyNew()
    local self = Event:new(setWrappingActiveEvent_mt);
    self.className = "setWrappingActiveEvent";
	
    return self;
end;

function setWrappingActiveEvent:new(object, active)
    local self = setWrappingActiveEvent:emptyNew();
    self.object = object;
	self.active = active;
	
    return self;
end;

function setWrappingActiveEvent:readStream(streamId, connection)
    local object = streamReadInt32(streamId);
    self.object = networkGetObject(object);
	self.active = streamReadBool(streamId);
	
    self:run(connection);
end;

function setWrappingActiveEvent:writeStream(streamId, connection)
    streamWriteInt32(streamId, networkGetObjectId(self.object));
	streamWriteBool(streamId, self.active);
end;

function setWrappingActiveEvent:run(connection)
	self.object:setWrappingActive(self.active, true);
	
	if not connection:getIsServer() then
		g_server:broadcastEvent(setWrappingActiveEvent:new(self.object, self.active), nil, connection, self.object);
	end;
end;

function setWrappingActiveEvent.sendEvent(object, active, noEventSend)
	if noEventSend == nil or noEventSend == false then
		if g_server ~= nil then
			g_server:broadcastEvent(setWrappingActiveEvent:new(object, active), nil, nil, object);
		else
			g_client:getServerConnection():sendEvent(setWrappingActiveEvent:new(object, active));
		end;
	end;
end;